package PublishingPagesCounter::CMS;

use strict;

use MT::Util qw( start_end_week );
use MT::PublishOption;
use MT::Promise qw(delay lazy force);

use Data::Dumper;

our @types = ('Individual','Page',
              'Category','Category-Yearly','Category-Monthly','Category-Weekly','Category-Daily',
              'Author','Author-Yearly','Author-Monthly','Author-Weekly','Author-Daily',
              'Yearly','Monthly','Weekly','Daily',
              'index');

sub publishing_pages {
    my ($app, $tmpl, $param) = @_;

    my $data = _get_rebuild_data($app);

    my $all;
    foreach my $type (@types) {
        $all += $param->{archives}->{$type} = $data->{$type} if $data->{$type};
    }
    $param->{all} = $all;
}

sub _get_timestamp {
    my ( $entry, $type ) = @_;
    my $timestamp;
    my $dummy;
    ( $timestamp = $entry->authored_on ) =~ s/^(\d{4})\d{10}$/${1}0000000000/ if $type =~ /Yearly/;
    ( $timestamp = $entry->authored_on ) =~ s/^(\d{4})(\d{2})\d{8}$/$1${2}00000000/ if $type =~ /Monthly/;
    ( $timestamp, $dummy ) = start_end_week( $entry->authored_on ) if $type =~ /Weekly/;
    ( $timestamp = $entry->authored_on ) =~ s/^(\d{4})(\d{2})(\d{2})\d{6}$/$1$2${3}000000/ if $type =~ /Daily/;
    $timestamp;
}

sub _get_rebuild_data {
    my $app = shift;

    my $blog_id = $app->blog->id;
    my $plugin = MT->component('PublishingPagesCounter');
    my $class = $app->model( 'template' ) or return $app->errtrans( 'Unsupported type' );

    my %param;
    my $dummy;

    foreach my $type (@types) {

        # entry
        if ( $type eq 'Individual' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Individual' ) != MT::PublishOption::DISABLED()) {
                $param{Individual} = MT::Entry->count( { blog_id => $blog_id, class => 'entry', status => MT::Entry::RELEASE() } );
            }
        }

        # page
        elsif ( $type eq 'Page' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Page' ) != MT::PublishOption::DISABLED()) {
                $param{Page} = MT::Entry->count( { blog_id => $blog_id, class => 'page', status => MT::Entry::RELEASE() } );
            }
        }

        # date
        elsif ( $type eq 'Yearly' || $type eq 'Monthly' || $type eq 'Weekly' || $type eq 'Daily') {
            if ( MT::PublishOption::archive_build_type( $blog_id, $type ) != MT::PublishOption::DISABLED()) {
                my $timestamp;
                my $list;
                my $iter = MT::Entry->load_iter( { blog_id => $blog_id, class => 'entry', status => MT::Entry::RELEASE() } );
                while ( my $entry = $iter->() ) {
                    $timestamp = _get_timestamp( $entry, $type );
                    $list->{ $timestamp } = 1;
                }
                $param{$type} = keys %$list;
            }
        }

        # category
        elsif ( $type eq 'Category' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Category' ) != MT::PublishOption::DISABLED()) {
                require MT::TemplateMap;
                $param{Category} = MT::Category->count( { blog_id => $blog_id, class => 'category' } );
            }
        }

        # category-date
        elsif ( $type eq 'Category-Yearly' || $type eq 'Category-Monthly' || $type eq 'Category-Weekly' || $type eq 'Category-Daily') {
            if ( MT::PublishOption::archive_build_type( $blog_id, $type ) != MT::PublishOption::DISABLED()) {
                my $timestamp;
                my $list;
                my @dates;

                my $iter = MT::Category->load_iter( { blog_id => $blog_id, class => 'category' } );
                my @cats;
                while ( my $cat = $iter->() ) {
                    my ( $terms, $args );
                    my ( $join_terms, $join_args );
                    $join_terms->{ category_id } = $cat->id;
                    $join_args->{ unique } = 1;
                    $terms->{ blog_id } = $blog_id;
                    $terms->{ status } = MT::Entry::RELEASE();
                    $args->{ join } = MT::Placement->join_on( 'entry_id', $join_terms );
                    $args->{ sort } = 'authored_on';
                    $args->{ direction } = 'descend';
                    my $iter2 = MT::Entry->load_iter( $terms, $args );

                    undef $list;
                    while ( my $entry = $iter2->() ) {
                        $timestamp = _get_timestamp( $entry, $type );
                        $list->{ $timestamp } = 1;
                    }
                    $param{$type} += keys %$list;
                }
            }
        }

        # author
        elsif ( $type eq 'Author' ) {
            if ( MT::PublishOption::archive_build_type( $blog_id, 'Author' ) != MT::PublishOption::DISABLED()) {
                my ( $args, $terms );
                $args->{sort} = 'id';
                $args->{unique} = 1;
                $terms->{status} = MT::Author::ACTIVE();
                require MT::Entry;
                $args->{join} = MT::Entry->join_on(
                    'author_id',
                    {   blog_id => $blog_id,
                        class   => 'entry',
                        status  => MT::Entry::RELEASE()
                    },
                    { unique => 1 }
                );
                $param{Author} = MT::Author->count( $terms, $args );
            }
        }

        # author-date
        elsif ( $type eq 'Author-Yearly' || $type eq 'Author-Monthly' || $type eq 'Author-Weekly' || $type eq 'Author-Daily') {
            if ( MT::PublishOption::archive_build_type( $blog_id, $type ) != MT::PublishOption::DISABLED()) {
                my $timestamp;
                my $list;
                my @dates;

                my ( $args, $terms );
                $args->{sort} = 'id';
                $args->{unique} = 1;
                $terms->{status} = MT::Author::ACTIVE();
                require MT::Entry;
                $args->{join} = MT::Entry->join_on(
                    'author_id',
                    {   blog_id => $blog_id,
                        class   => 'entry',
                        status  => MT::Entry::RELEASE()
                    },
                    { unique => 1 }
                );
                my $iter = MT::Author->load_iter( $terms, $args );
                my @authors;
                while ( my $author = $iter->() ) {
                    my ( $terms, $args );
                    $terms->{ status } = MT::Entry::RELEASE();
                    $terms->{ blog_id } = $blog_id;
                    $terms->{ author_id } = $author->id;
                    $args->{ sort } = 'authored_on';
                    $args->{ direction } = 'descend';
                    my $iter2 = MT::Entry->load_iter( $terms, $args );

                    undef $list;
                    while ( my $entry = $iter2->() ) {
                        $timestamp = _get_timestamp( $entry, $type );
                        $list->{ $timestamp } = 1;
                    }
                    $param{$type} += keys %$list;
                }
            }
        }

        # index
        elsif ( $type eq 'index' ) {
            $param{index} = MT::Template->count( { blog_id => $blog_id, type => 'index', rebuild_me => [1, 2] } );
        }
    }

    return \%param;
}


1;
